﻿using System;
using System.Text;

namespace Plus.HabboHotel.Rooms
{
    public class DynamicRoomModel
    {
        public bool ClubOnly { get; set; }
		public int DoorX { get; set; }
		public int DoorY { get; set; }
		public double DoorZ { get; set; }
		public int DoorOrientation { get; set; }
		public string Heightmap { get; set; }
        public int MapSizeX { get; set; }
		public int MapSizeY { get; set; }
		public short[,] SqFloorHeight { get; set; }
		public byte[,] SqSeatRot { get; set; }
		public SquareState[,] SqState { get; set; }

		private RoomModel _staticModel;
        private string _relativeHeightmap = null;

        public DynamicRoomModel(RoomModel model)
		{
			ClubOnly = _staticModel.ClubOnly;
			DoorX = _staticModel.DoorX;
            DoorY = _staticModel.DoorY;
            DoorZ = _staticModel.DoorZ;
            DoorOrientation = _staticModel.DoorOrientation;
            Heightmap = _staticModel.Heightmap;
            MapSizeX = _staticModel.MapSizeX;
            MapSizeY = _staticModel.MapSizeY;
			SqFloorHeight = new short[MapSizeX, MapSizeY];
			SqSeatRot = new byte[MapSizeX, MapSizeY];
			SqState = new SquareState[MapSizeX, MapSizeY];
			_staticModel = model;

			for (int y = 0; y < MapSizeY; y++)
            {
                for (int x = 0; x < MapSizeX; x++)
                {
                    if (x > (_staticModel.MapSizeX - 1) || y > (_staticModel.MapSizeY - 1))
                    {
                        SqState[x, y] = SquareState.Blocked;
                    }
                    else
                    {
                        SqState[x, y] = _staticModel.SqState[x, y];
                        SqFloorHeight[x, y] = _staticModel.SqFloorHeight[x, y];
                        SqSeatRot[x, y] = _staticModel.SqSeatRot[x, y];
                    }
                }
            }

			StringBuilder floorMap = new StringBuilder();

            for (int y = 0; y < MapSizeY; y++)
            {
                for (int x = 0; x < MapSizeX; x++)
                {
                    if (x == DoorX && y == DoorY)
                    {
                        floorMap.Append(DoorZ > 9 ? ((char)(87 + DoorZ)).ToString() : DoorZ.ToString());
                        continue;
                    }

                    if (SqState[x, y] == SquareState.Blocked)
                    {
                        floorMap.Append('x');
                        continue;
                    }

                    double Height = SqFloorHeight[x, y];

                    floorMap.Append(Height > 9 ? ((char)(87 + Height)).ToString() : Height.ToString());
                }
                floorMap.Append(Convert.ToChar(13));
            }

            this._relativeHeightmap = floorMap.ToString();
        }

        public void refreshArrays()
        {
            var newSqState = new SquareState[MapSizeX + 1, MapSizeY + 1];
            var newSqFloorHeight = new short[MapSizeX + 1, MapSizeY + 1];
            var newSqSeatRot = new byte[MapSizeX + 1, MapSizeY + 1];

            for (int y = 0; y < MapSizeY; y++)
            {
                for (int x = 0; x < MapSizeX; x++)
                {
                    if (x > (_staticModel.MapSizeX - 1) || y > (_staticModel.MapSizeY - 1))
                    {
                        newSqState[x, y] = SquareState.Blocked;
                    }
                    else
                    {
                        newSqState[x, y] = SqState[x, y];
                        newSqFloorHeight[x, y] = SqFloorHeight[x, y];
                        newSqSeatRot[x, y] = SqSeatRot[x, y];
                    }
                }
            }

            SqState = newSqState;
            SqFloorHeight = newSqFloorHeight;
            SqSeatRot = newSqSeatRot;
        }

        public string GetRelativeHeightmap()
        {
            return this._relativeHeightmap;
        }

        public void AddX()
        {
            MapSizeX++;
            refreshArrays();
        }

        public void OpenSquare(int x, int y, double z)
        {
            if (z > 9)
                z = 9;
            if (z < 0)
                z = 0;
            SqFloorHeight[x, y] = (short)z;
            SqState[x, y] = SquareState.Open;
        }

        public void AddY()
        {
            MapSizeY++;
            refreshArrays();
        }

        public bool DoorIsValid()
        {
            if (DoorX > SqFloorHeight.GetUpperBound(0) || DoorY > SqFloorHeight.GetUpperBound(1))
                return false;
            else
                return true;
        }

        public void SetMapsize(int x, int y)
        {
            MapSizeX = x;
            MapSizeY = y;
            refreshArrays();
        }

        public void Destroy()
        {
            Array.Clear(SqState, 0, SqState.Length);
            Array.Clear(SqFloorHeight, 0, SqFloorHeight.Length);
            Array.Clear(SqSeatRot, 0, SqSeatRot.Length);

            _staticModel = null;
			_relativeHeightmap = null;
            Heightmap = null;
            SqState = null;
            SqFloorHeight = null;
            SqSeatRot = null;
        }
    }
}