﻿using System;
using Plus.Core;

namespace Plus.HabboHotel.Rooms
{
	public class RoomModel : IDisposable
	{
		public bool ClubOnly;
		public int DoorX;
		public int DoorY;
		public double DoorZ;
		public int DoorOrientation;
		public string Heightmap;
		public int MapSizeX;
		public int MapSizeY;
		public short[,] SqFloorHeight;
		public byte[,] SqSeatRot;
		public SquareState[,] SqState;
		public int WallHeight;

		public RoomModel(int doorX, int doorY, double doorZ, int doorOrientation, string heightmap, bool clubOnly, int wallHeight)
		{
			this.DoorX = doorX;
			this.DoorY = doorY;
			this.DoorZ = doorZ;
			this.DoorOrientation = doorOrientation;

			this.WallHeight = wallHeight;

			this.Heightmap = heightmap.ToLower();

			string[] tmpHeightmap = heightmap.Split(Convert.ToChar(13));

			this.MapSizeX = tmpHeightmap[0].Length;
			this.MapSizeY = tmpHeightmap.Length;
			this.ClubOnly = clubOnly;

			SqState = new SquareState[MapSizeX, MapSizeY];
			SqFloorHeight = new short[MapSizeX, MapSizeY];
			SqSeatRot = new byte[MapSizeX, MapSizeY];

			try
			{
				for (int y = 0; y < MapSizeY; y++)
				{
					string line = tmpHeightmap[y];
					line = line.Replace("\r", "");
					line = line.Replace("\n", "");

					int x = 0;
					foreach (char square in line)
					{
						if (square == 'x')
						{
							SqState[x, y] = SquareState.Blocked;
						}
						else
						{
							SqState[x, y] = SquareState.Open;
							SqFloorHeight[x, y] = parse(square);
						}
						x++;
					}
				}
			}
			catch (Exception e)
			{
				ExceptionLogger.LogException(e);
			}
		}

		public static short parse(char input)
		{

			switch (input)
			{
				case '0':
					return 0;
				case '1':
					return 1;
				case '2':
					return 2;
				case '3':
					return 3;
				case '4':
					return 4;
				case '5':
					return 5;
				case '6':
					return 6;
				case '7':
					return 7;
				case '8':
					return 8;
				case '9':
					return 9;
				case 'a':
					return 10;
				case 'b':
					return 11;
				case 'c':
					return 12;
				case 'd':
					return 13;
				case 'e':
					return 14;
				case 'f':
					return 15;
				case 'g':
					return 16;
				case 'h':
					return 17;
				case 'i':
					return 18;
				case 'j':
					return 19;
				case 'k':
					return 20;
				case 'l':
					return 21;
				case 'm':
					return 22;
				case 'n':
					return 23;
				case 'o':
					return 24;
				case 'p':
					return 25;
				case 'q':
					return 26;
				case 'r':
					return 27;
				case 's':
					return 28;
				case 't':
					return 29;
				case 'u':
					return 30;
				case 'v':
					return 31;
				case 'w':
					return 32;

				default:
					throw new FormatException("The input was not in a correct format: input must be between (0-k)");
			}
		}

		public void Dispose()
		{
			Heightmap = null;
			SqState = null;
			SqFloorHeight = null;
			SqSeatRot = null;
		}
	}
}